@@ -32,6 +32,7 @@ struct CmdPetitionPosixSockets: public CmdPetition
 
     virtual ~CmdPetitionPosixSockets()
     {
+        shutdown(outSocket, SHUT_RDWR);
         close(outSocket);
     }
 
@@ -58,6 +59,7 @@ private:
     std::mutex mtx;
     std::mutex informerMutex;
 
+    void sendPartialOutputImpl(CmdPetition *inf, char *s, size_t size, bool binaryContents, bool sendAsError);
 public:
     ComunicationsManagerFileSockets();
 
@@ -81,6 +83,8 @@ public:
 
     void sendPartialOutput(CmdPetition *inf, OUTSTRING *s) override;
     void sendPartialOutput(CmdPetition *inf, char *s, size_t size, bool binaryContents = false) override;
+    void sendPartialError(CmdPetition *inf, OUTSTRING *s) override;
+    void sendPartialError(CmdPetition *inf, char *s, size_t size, bool binaryContents = false) override;
 
     int informStateListener(CmdPetition *inf, const std::string &s) override;
 
