@@ -863,37 +863,62 @@ void printPercentageLineCerr(const char *title, long long completed, long long t
     }
 }
 
-int getFlag(map<string, int> *flags, const char * optname)
+int getFlag(const map<string, int> *flags, const char * optname)
 {
-    return flags->count(optname) ? ( *flags )[optname] : 0;
+    auto i = flags->find(optname);
+
+    if (i != flags->end())
+        return i->second;
+
+    return 0;
 }
 
-string getOption(map<string, string> *cloptions, const char * optname, string defaultValue)
+string getOption(const map<string, string> *cloptions, const char * optname, string defaultValue)
 {
-    return cloptions->count(optname) ? ( *cloptions )[optname] : defaultValue;
+    auto i = cloptions->find(optname);
+
+    if (i != cloptions->end())
+        return i->second;
+
+    return defaultValue;
 }
 
 std::optional<string> getOptionAsOptional(const map<string, string>& cloptions, const char * optname)
 {
     if (cloptions.find(optname) == cloptions.end())
     {
-        return {};
+        return std::nullopt;
     }
     return cloptions.at(optname);
 }
 
-int getintOption(map<string, string> *cloptions, const char * optname, int defaultValue)
+int getintOption(const map<string, string> *cloptions, const char * optname, int defaultValue)
 {
-    if (cloptions->count(optname))
+    auto i = cloptions->find(optname);
+
+    auto result = defaultValue;
+
+    if (i != cloptions->end())
+        istringstream(i->second) >> result;
+
+    return result;
+}
+
+std::optional<int> getIntOptional(const std::map<std::string, std::string>& cloptions, const char* optName)
+{
+    auto it = cloptions.find(optName);
+    if (it == cloptions.end())
     {
-        int i = defaultValue;
-        istringstream is(( *cloptions )[optname]);
-        is >> i;
-        return i;
+        return std::nullopt;
     }
-    else
+
+    try
     {
-        return defaultValue;
+        return std::stoi(it->second);
+    }
+    catch (...)
+    {
+        return std::nullopt;
     }
 }
 
