@@ -28,6 +28,12 @@
 
 namespace megacmd {
 
+// Provide a helpful error message for the provided error, setting the current error code in case of an error.
+std::string formatErrorAndMaySetErrorCode(const mega::MegaError &error);
+bool checkNoErrors(int errorCode, const std::string &message = "");
+bool checkNoErrors(mega::MegaError *error, const std::string &message = "", mega::SyncError syncError = mega::SyncError::NO_SYNC_ERROR);
+bool checkNoErrors(::mega::SynchronousRequestListener *listener, const std::string &message = "", mega::SyncError syncError = mega::SyncError::NO_SYNC_ERROR);
+
 #ifdef WIN32
 inline ::mega::SimpleLogger &operator<<(::mega::SimpleLogger& sl, const fs::path& path)
 {
@@ -83,6 +89,7 @@ public:
     virtual const LoggedStream& operator<<(unsigned int v) const = 0;
     virtual const LoggedStream& operator<<(long unsigned int v) const = 0;
     virtual const LoggedStream& operator<<(long long int v) const = 0;
+    virtual const LoggedStream& operator<<(long long unsigned int v) const = 0;
     virtual const LoggedStream& operator<<(std::ios_base v) const = 0;
     virtual const LoggedStream& operator<<(std::ios_base *v) const = 0;
 
@@ -101,6 +108,7 @@ public:
 #ifdef _WIN32
     const LoggedStream& operator<<(std::wstring v) const override { return *this; }
 #endif
+
     const LoggedStream& operator<<(std::string v) const override { return *this; }
     const LoggedStream& operator<<(BinaryStringView v) const override { return *this; }
     const LoggedStream& operator<<(std::string_view v) const override { return *this; }
@@ -108,6 +116,7 @@ public:
     const LoggedStream& operator<<(unsigned int v) const override { return *this; }
     const LoggedStream& operator<<(long unsigned int v) const override { return *this; }
     const LoggedStream& operator<<(long long int v) const override { return *this; }
+    const LoggedStream& operator<<(long long unsigned int v) const override { return *this; }
     const LoggedStream& operator<<(std::ios_base v) const override { return *this; }
     const LoggedStream& operator<<(std::ios_base *v) const override { return *this; }
 
@@ -168,6 +177,7 @@ public:
     virtual const LoggedStream& operator<<(unsigned int v) const override { *out << v;return *this; }
     virtual const LoggedStream& operator<<(long unsigned int v) const override { *out << v;return *this; }
     virtual const LoggedStream& operator<<(long long int v) const override { *out << v;return *this; }
+    virtual const LoggedStream& operator<<(long long unsigned int v) const override { *out << v;return *this; }
     virtual const LoggedStream& operator<<(std::ios_base v) const override { *out << &v;return *this; }
     virtual const LoggedStream& operator<<(std::ios_base *v) const override { *out << v;return *this; }
 
@@ -206,10 +216,11 @@ public:
     virtual const LoggedStream& operator<<(unsigned int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialOutput(inf, &s); return *this; }
     virtual const LoggedStream& operator<<(long unsigned int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialOutput(inf, &s); return *this; }
     virtual const LoggedStream& operator<<(long long int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialOutput(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(long long unsigned int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialOutput(inf, &s); return *this; }
     virtual const LoggedStream& operator<<(std::ios_base v) const override { *out << &v;return *this; }
     virtual const LoggedStream& operator<<(std::ios_base *v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialOutput(inf, &s); return *this; }
 
-    LoggedStream const& operator<<(OUTSTREAMTYPE& (*F)(OUTSTREAMTYPE&)) const
+    LoggedStream const& operator<<(OUTSTREAMTYPE& (*F)(OUTSTREAMTYPE&)) const override
     {
         OUTSTRINGSTREAM os; os << F; OUTSTRING s = os.str(); cm->sendPartialOutput(inf, &s);
         return *this;
@@ -222,9 +233,50 @@ protected:
     CmdPetition *inf;
 };
 
+
+class LoggedStreamPartialErrors : public LoggedStream
+{
+public:
+    LoggedStreamPartialErrors(ComunicationsManager *_cm, CmdPetition *_inf) : cm(_cm), inf(_inf) {}
+    virtual bool isClientConnected() override { return inf && !inf->clientDisconnected; }
+
+    virtual const LoggedStream& operator<<(const char& v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(const char* v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+
+#ifdef _WIN32
+    virtual const LoggedStream& operator<<(std::wstring v) const override { cm->sendPartialError(inf, &v); return *this; }
+    virtual const LoggedStream& operator<<(std::string v) const override { cm->sendPartialError(inf, (char *)v.data(), v.size()); return *this; }
+#else
+    virtual const LoggedStream& operator<<(std::string v) const override { cm->sendPartialError(inf, &v); return *this; }
+#endif
+    virtual const LoggedStream& operator<<(BinaryStringView v) const override { cm->sendPartialError(inf, (char*) v.get().data(), v.get().size(), true); return *this; }
+    virtual const LoggedStream& operator<<(std::string_view v) const override { cm->sendPartialError(inf, (char*) v.data(), v.size()); return *this; }
+
+    virtual const LoggedStream& operator<<(int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(unsigned int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(long unsigned int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(long long int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(long long unsigned int v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+    virtual const LoggedStream& operator<<(std::ios_base v) const override { *out << &v;return *this; }
+    virtual const LoggedStream& operator<<(std::ios_base *v) const override { OUTSTRINGSTREAM os; os << v; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s); return *this; }
+
+    LoggedStream const& operator<<(OUTSTREAMTYPE& (*F)(OUTSTREAMTYPE&)) const override
+    {
+        OUTSTRINGSTREAM os; os << F; OUTSTRING s = os.str(); cm->sendPartialError(inf, &s);
+        return *this;
+    }
+
+    virtual ~LoggedStreamPartialErrors() = default;
+
+protected:
+    ComunicationsManager *cm;
+    CmdPetition *inf;
+};
+
 struct ThreadData
 {
     LoggedStream *mOutStream = &Instance<DefaultLoggedStream>::Get().getLoggedStream();
+    LoggedStream *mErrStream = &Instance<DefaultLoggedStream>::Get().getLoggedStream();
     int mLogLevel = -1;
     int mOutCode = 0;
     CmdPetition *mCmdPetition = nullptr;
@@ -236,12 +288,13 @@ const char* getCommandPrefixBasedOnMode();
 bool isCurrentThreadInteractive();
 
 inline LoggedStream &getCurrentThreadOutStream()  { return *getCurrentThreadData().mOutStream; }
+inline LoggedStream &getCurrentThreadErrStream()  { return *getCurrentThreadData().mErrStream; }
 inline int getCurrentThreadLogLevel()             { return getCurrentThreadData().mLogLevel; }
 inline int getCurrentThreadOutCode()              { return getCurrentThreadData().mOutCode; }
 inline CmdPetition *getCurrentThreadCmdPetition() { return getCurrentThreadData().mCmdPetition; }
 inline bool isCurrentThreadCmdShell()             { return getCurrentThreadData().mIsCmdShell; }
 
-void setCurrentThreadOutStream(LoggedStream &outStream);
+void setCurrentThreadOutStreams(LoggedStream &outStream, LoggedStream &errStream);
 void setCurrentThreadOutCode(int outCode);
 void setCurrentThreadLogLevel(int logLevel);
 void setCurrentThreadCmdPetition(CmdPetition *cmdPetition);
