@@ -469,13 +469,14 @@ bool outputtobinaryorconsole(void)
     }
 }
 
-int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::string (*readresponse)(const char *), OUTSTREAMTYPE &output, bool interactiveshell, wstring wcommand)
+int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::string (*readresponse)(const char *), OUTSTREAMTYPE &output, OUTSTREAMTYPE &errorOutput, bool interactiveshell, wstring wcommand)
 {
     HANDLE theNamedPipe = createNamedPipe(0,command.compare(0,4,"exit")
                                           && command.compare(0,4,"quit")
                                           && command.compare(0,7,"sendack"));
     if (!namedPipeValid(theNamedPipe))
     {
+        cerr << "MegaCmdShellCommunicationsNamedPipes::executeCommand ERROR, named pipe not valid: " << theNamedPipe << " errno = "  << ERRNO << endl;
         return -1;
     }
 
@@ -521,6 +522,7 @@ int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::st
     HANDLE newNamedPipe = createNamedPipe(receiveNamedPipeNum);
     if (!namedPipeValid(newNamedPipe))
     {
+        cerr << "MegaCmdShellCommunicationsNamedPipes::executeCommand ERROR, receive named pipe not valid: " << receiveNamedPipeNum << " pipe= " << newNamedPipe << " errno = "  << ERRNO << endl;
         return -1;
     }
 
@@ -540,10 +542,13 @@ int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::st
     bool binaryoutput = isCat && redirectedstdout;
     bool shouldPrintAdditionalLine = false;
 
-    while (outcode == MCMD_REQCONFIRM || outcode == MCMD_REQSTRING || outcode == MCMD_PARTIALOUT)
+    while (outcode == MCMD_REQCONFIRM || outcode == MCMD_REQSTRING || outcode == MCMD_PARTIALOUT || outcode == MCMD_PARTIALERR)
     {
-        if (outcode == MCMD_PARTIALOUT)
+        if (outcode == MCMD_PARTIALOUT || outcode == MCMD_PARTIALERR)
         {
+            bool useErrorStream = outcode == MCMD_PARTIALERR;
+            auto &partialOutputStream = useErrorStream ? errorOutput : output;
+
             size_t partialoutsize;
             if (!ReadFile(newNamedPipe, (char *)&partialoutsize, sizeof(partialoutsize),&n, NULL))
             {
@@ -570,7 +575,7 @@ int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::st
                     readok = ReadFile(newNamedPipe, buffer, DWORD(std::min(BUFFERSIZE,partialoutsize)),&n,NULL);
                     if (readok)
                     {
-                        if (binaryoutput)
+                        if (binaryoutput && !useErrorStream)
                         {
                             Instance<WindowsConsoleController>::Get().enableInterceptors(false);
                             std::cout << string(buffer,n) << flush;
@@ -582,13 +587,13 @@ int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::st
 
                             wstring wbuffer;
                             stringtolocalw((const char*)&buffer,&wbuffer);
-                            output << wbuffer << flush;
+                            partialOutputStream << wbuffer << flush;
                         }
                         partialoutsize-=n;
                     }
                 } while(n != 0 && partialoutsize && n !=SOCKET_ERROR);
 
-                if (isCat && !binaryoutput && interactiveshell)
+                if (isCat && !binaryoutput && interactiveshell && !useErrorStream)
                 {
                     shouldPrintAdditionalLine = true;
                 }
@@ -705,15 +710,27 @@ int MegaCmdShellCommunicationsNamedPipes::executeCommand(string command, std::st
 
 int MegaCmdShellCommunicationsNamedPipes::listenToStateChanges(int receiveNamedPipeNum, StateChangedCb_t statechangehandle)
 {
-    HANDLE newNamedPipe = createNamedPipe(receiveNamedPipeNum);
-    if (!namedPipeValid(newNamedPipe))
     {
-        return -1;
+        std::lock_guard<std::mutex> g(mStateListenerNamedPipeMutex);
+        mStateListenerNamedPipeHandle = createNamedPipe(receiveNamedPipeNum);
+        if (!namedPipeValid(mStateListenerNamedPipeHandle))
+        {
+            return -1;
+        }
+        assert(!mStateListenerNamedPipeResetPromise);
+        mStateListenerNamedPipeResetPromise.reset(new std::promise<void>());
     }
 
-    ScopeGuard g([this, &newNamedPipe]()
+    ScopeGuard g([this]()
     {
-        closeNamedPipe(newNamedPipe);
+        if (namedPipeValid(mStateListenerNamedPipeHandle))
+        {
+            closeNamedPipe(mStateListenerNamedPipeHandle);
+            std::lock_guard<std::mutex> g(mStateListenerNamedPipeMutex);
+            mStateListenerNamedPipeHandle = INVALID_HANDLE_VALUE;
+            auto promise = std::move(mStateListenerNamedPipeResetPromise);
+            promise->set_value();
+        }
     });
 
     int timeout_notified_server_might_be_down = 0;
@@ -726,7 +743,7 @@ int MegaCmdShellCommunicationsNamedPipes::listenToStateChanges(int receiveNamedP
         DWORD n;
         bool readok;
         do{
-            readok = ReadFile(newNamedPipe, buffer, BUFFERSIZE, &n, NULL);
+            readok = ReadFile(mStateListenerNamedPipeHandle, buffer, BUFFERSIZE, &n, NULL);
             if (readok)
             {
                 buffer[n]='\0';
@@ -736,17 +753,13 @@ int MegaCmdShellCommunicationsNamedPipes::listenToStateChanges(int receiveNamedP
 
         if (!readok)
         {
-            if (ERRNO == ERROR_BROKEN_PIPE)
+            if ((mStopListener || mUpdating) && (ERRNO == ERROR_BROKEN_PIPE || ERRNO == ERROR_OPERATION_ABORTED))
             {
-                if (!mStopListener && !mUpdating)
-                {
-                    cerr << "ERROR reading output (state change): The sever problably exited."<< endl;
-                }
-            }
-            else
-            {
-                cerr << "ERROR reading output (state change): " << ERRNO << endl;
+                // expected errors after shutdown:
+                return 0;
             }
+
+            cerr << "ERROR reading output (state change): " << ERRNO << endl;
             return -1;
         }
 
@@ -805,8 +818,32 @@ void MegaCmdShellCommunicationsNamedPipes::setResponseConfirmation(bool confirma
 
 void MegaCmdShellCommunicationsNamedPipes::triggerListenerThreadShutdown()
 {
+    std::unique_lock<std::mutex> g(mStateListenerNamedPipeMutex);
     // this would cause the wake of the listener thread:
-    executeCommand("sendack");
+    if (namedPipeValid(mStateListenerNamedPipeHandle))
+    {
+        std::optional<std::future<void>> fut;
+        if (mStateListenerNamedPipeResetPromise)
+        {
+            fut = mStateListenerNamedPipeResetPromise->get_future();
+        }
+        CancelIoEx(mStateListenerNamedPipeHandle, NULL);
+        // if we were extremely unlucky and above cancel occurred when still not in ReadFile, below wait would fail. If that's the case,
+        // a second attempt should work
+        g.unlock();
+
+        if (fut && fut->wait_for(std::chrono::milliseconds(200)) == std::future_status::ready)
+        {
+            return;
+        }
+
+        g.lock();
+        if (namedPipeValid(mStateListenerNamedPipeHandle))
+        {
+            CancelIoEx(mStateListenerNamedPipeHandle, NULL);
+        }
+
+    };
 }
 
 MegaCmdShellCommunicationsNamedPipes::~MegaCmdShellCommunicationsNamedPipes()
